@@ -20,11 +20,18 @@ window.vm = new Vue({
             config: window.config,
             selectedPlugin: undefined,
             now: new Date(),
+            fullscreen: false,
         };
     },
 
+    methods: {
+        toggleFullScreen: function() {
+            toggleFullScreen();
+        },
+    },
+
     created: function() {
-        m = window.location.href.match('#([a-zA-Z0-9._]+)$');
+        let m = window.location.href.match('#([a-zA-Z0-9._]+)$');
         if (m) {
             this.selectedPlugin = m[1];
         }
